@@ -2,41 +2,14 @@ import importlib
 import json
 import sys
 import os
+import types
 import collections
 import warnings
 from pkg_resources import parse_version
 import operator
 
-class Namespace(object):
-    """
-    dummy class to hold namespaces during plugin loading
-    """
-    pass
 
-def expand_version_requierment(version):
-        """
-        Takes a string of one of the following forms:
 
-        "" -> no version requierment
-        "*" -> no version requierment
-        "plugin_name" -> spesfic plugin no version requierment
-        "plugin_name:version_ranges" -> spesfic plugin version matches requierments
-
-        and returns one of the following:
-
-        ("", "") -> no version requierment
-        ("plugin_name", "") -> plugin_name but no version requierment
-        ("plugin_name", "verison_ranges")
-        """
-        if version == "*" or version == "":
-            return ("", "")
-        elif ":" in version:
-            parts = version.split(":")
-            if len(parts) != 2:
-                raise RuntimeError("Version requierments can only contain at most 2 parts, one plugin_name and one set of version requierments, the parts seperated by a ':'")
-            return (parts[0], parts[1] )
-        else:
-            return (version,  "")
 
 
 
@@ -60,35 +33,66 @@ class Plugin(object):
         if 'name' in config:
             self.name = config['name'].strip()
         else:
-            raise RuntimeError("Plugin as %s does not have a name" % path)
-        if 'author' in config:
+            raise RuntimeError("Plugin as '%s' does not have a name string" % path)
+        if 'author' in config and isinstance(config['author'], str):
             self.author = config['author'].strip()
         else:
-            raise RuntimeError("Plugin as %s does not have an author" % path)
+            raise RuntimeError("Plugin as '%s' does not have a author string" % path)
         if 'version' in config:
             #store both the original version string and a parsed version that can be compaired accurately
             self.version = (config['version'].strip(), parse_version(config['version'].strip()))
         else:
-            raise RuntimeError("Plugin as %s does not have a version" % path)
+            raise RuntimeError("Plugin at '%s' does not have a version" % path)
         if 'file' in config:
             self.file = config['file'].strip()
         else:
-            raise RuntimeError("Plugin as %s does not have a plugin file spesified" % path)
+            raise RuntimeError("Plugin as '%s' does not have a plugin file spesified" % path)
         if 'consumes' in config and isinstance(config['consumes'], collections.Mapping):
             self.consumes = config['consumes']
         else:
-            raise RuntimeError("Plugin as %s does not have a mapping of consumed components to plugin versions" % path)
+            raise RuntimeError("Plugin at '%s'does not have a mapping of consumed components to plugin versions" % path)
         if 'provides' in config and isinstance(config['provides'], collections.Mapping):
             self.provides = config['provides']
         else:
-            raise RuntimeError("Plugin as %s does not have a mapping of provided components to version postfixes" % path)
+            raise RuntimeError("Plugin at '%s' does not have a mapping of provided components to version postfixes" % path)
+        if ('mode' in config and (config['mode'].lower() == 'import' or config['mode'] == 'exec' )):
+            self.mode = config['mode']
+            if self.mode == 'import' and not Plugin.supports_import_mode():
+                self.mode = 'exec'
+                warnings.warn(RuntimeWarning("Plugin at '%s' has set 'import' mode but this mode is only suported in python 3.4 and up, defaulting to 'exec' mode" % path))
+        elif not ('mode' in config):
+            self.mode = 'import' if Plugin.supports_import_mode() else 'exec'
+        else:
+            raise RuntimeError("Plugin at '%s' has bad mode, 'import' and 'exec' allowed" % path)
         self.path = path
 
+    @staticmethod
+    def supports_import_mode():
+        return sys.hexversion >= 0x030400F0
+
     def load(self):
         """loads the plugin file and returns the resulting module"""
         filepath = os.path.join(self.path, self.file)
-        spec = importlib.util.spec_from_file_location(self.name, filepath)
-        plugin = spec.loader.load_module()
+        if self.mode == 'import':
+            #import mode can handle situations where the file isn't a python source file, for example a compiled pyhton module in the form of a .pyd or .so
+            try:
+                sys.path.insert(0, self.path)
+                spec = importlib.util.spec_from_file_location(self.name, filepath)
+                plugin = spec.loader.load_module()
+                sys.path.remove(self.path)
+            except Exception as err:
+                raise RuntimeError("Plugin '%s' at '%s' failed to load" % (self.name, self.path)) from err
+        elif self.mode =='exec':
+            #exec mode requieres the file to be raw python
+            try:
+                plugin = types.ModuleType(self.name)
+                sys.path.insert(0, self.path)
+                with open(filepath) as f:
+                    code = compile(f.read(), self.file, 'exec')
+                    exec(code, plugin.__dict__)
+                sys.path.remove(self.path)
+            except Exception as err:
+                raise RuntimeError("Plugin '%s' at '%s' failed to load" % (self.name, self.path)) from err
         return plugin
 
     def get_version_string(self):
@@ -285,6 +289,32 @@ class System(object):
         else:
             self._add_plugin(os.path.dirname(path))
 
+    @staticmethod
+    def expand_version_requierment(version):
+        """
+        Takes a string of one of the following forms:
+
+        "" -> no version requierment
+        "*" -> no version requierment
+        "plugin_name" -> spesfic plugin no version requierment
+        "plugin_name:version_ranges" -> spesfic plugin version matches requierments
+
+        and returns one of the following:
+
+        ("", "") -> no version requierment
+        ("plugin_name", "") -> plugin_name but no version requierment
+        ("plugin_name", "verison_ranges")
+        """
+        if version == "*" or version == "":
+            return ("", "")
+        elif ":" in version:
+            parts = version.split(":")
+            if len(parts) != 2:
+                raise RuntimeError("Version requierments can only contain at most 2 parts, one plugin_name and one set of version requierments, the parts seperated by a ':'")
+            return (parts[0], parts[1] )
+        else:
+            return (version,  "")
+
     def resolve_highest_match(self, component, plugin, version):
         """
         resolves the latest version of a component with requierments, passing empty strings means no requierments
@@ -319,7 +349,7 @@ class System(object):
         if not plugin in self.components[component]:
             raise RuntimeError("Component '%s' is not provided by 'plugin' %s" % (component, plugin))
 
-        # our requerments might pass if we satify one of a number of version ranges
+        # our requierments might pass if we satify one of a number of version ranges
         version_ranges = []
         if " || " in version:
             # there are two or more version ranges, either could be satisfied
@@ -450,6 +480,59 @@ class System(object):
             else:
                 yield (plugin, version)
 
+    def _load_component(self, component, plugin, version, requesting=None):
+
+        # be sure not to load things twice, but besure the components is loaded and saved
+        if not component in self.loaded_components:
+            self.loaded_components[component] = {}
+        if not plugin in self.loaded_components[component]:
+            self.loaded_components[component][plugin] = {}
+        if not version in self.loaded_components[component][plugin]:
+
+            plugin_obj = self._load_plugin(plugin, version, requesting=requesting, component=component)
+
+            access_name = component
+            if plugin in self.postfix_mappings and component in self.postfix_mappings[plugin] and version in self.postfix_mappings[plugin][component]:
+                access_name = self.postfix_mappings[plugin][component][version]
+            if not hasattr(plugin_obj, access_name):
+                raise RuntimeError("Plugin '%s:%s' does not have name '%s'" % (plugin, version, access_name) )
+
+            self.loaded_components[component][plugin][version] = getattr(plugin_obj, access_name)
+
+            #record the use of this component, perhaps so the users can save the configuration
+            if not component in self.useing:
+                self.useing[component] = {}
+            if not plugin in self.useing[component]:
+                self.useing[component][plugin] = []
+            if not version in self.useing[component][plugin]:
+                self.useing[component][plugin].append(version)
+
+            self.fire_event('component_loaded', component, requesting, plugin + ":" + version)
+
+        component_obj = self.loaded_components[component][plugin][version]
+        return component_obj
+
+    def _load_plugin(self, plugin, version, requesting=None, component=None):
+        #we dont want to load a plugin twice just becasue it provides more than one component, save previouly loaded plugins
+        if not plugin in self.loaded_plugins:
+            self.loaded_plugins[plugin] = {}
+        if not version in self.loaded_plugins[plugin]:
+            #create a blank module namespace to attach our equired components
+            consumes = types.ModuleType("PyitectConsumes")
+
+            plugin_cfg = self.plugins[plugin][version]
+
+            for component_req in plugin_cfg.consumes.keys():
+                setattr(consumes, component_req, self.load(component_req, plugin_cfg.consumes, requesting=plugin_cfg.get_version_string()))
+
+            sys.modules["PyitectConsumes"] = consumes
+            self.loaded_plugins[plugin][version] = plugin_cfg.load()
+            del sys.modules["PyitectConsumes"]
+            self.fire_event('plugin_loded', plugin_cfg.get_version_string(), requesting, component)
+
+        plugin_obj = self.loaded_plugins[plugin][version]
+        return plugin_obj
+
     def load(self, component, requierments=None, requesting=None):
         """
         processes loading and returns the component by name,
@@ -471,57 +554,15 @@ class System(object):
         if not requierments is None:
             reqs.update(requierments)
 
-
         # update the plugin and version requierments if they exist
         if component in reqs:
-            plugin_req, version_req = expand_version_requierment(reqs[component])
+            plugin_req, version_req = System.expand_version_requierment(reqs[component])
         else:
             warnings.warn(RuntimeWarning("Component '%s' has no default provided, defaulting to alphabetical order" % component))
 
         # get the plugin and version to load
         plugin, version = self.resolve_highest_match(component, plugin_req, version_req)
 
-        # be sure not to load things twice, but besure the components is loaded and saved
-        if not component in self.loaded_components:
-            self.loaded_components[component] = {}
-        if not plugin in self.loaded_components[component]:
-            self.loaded_components[component][plugin] = {}
-        if not version in self.loaded_components[component][plugin]:
+        component = self._load_component(component, plugin, version)
 
-            #we dont want to load a plugin twice just becasue it provides more than one component, save previouly loaded plugins
-            if not plugin in self.loaded_plugins:
-                self.loaded_plugins[plugin] = {}
-            if not version in self.loaded_plugins[plugin]:
-                #we'll use this to store the needed components for the plugin we'll be loading
-                consumes = Namespace.__new__(Namespace)
-
-                plugin_cfg = self.plugins[plugin][version]
-
-                for component_req in plugin_cfg.consumes.keys():
-                    setattr(consumes, component_req, self.load(component_req, plugin_cfg.consumes, requesting=plugin_cfg.get_version_string()))
-
-                sys.modules["PyitectConsumes"] = consumes
-                self.loaded_plugins[plugin][version] = plugin_cfg.load()
-                del sys.modules["PyitectConsumes"]
-                self.fire_event('plugin_loded', plugin_cfg.get_version_string(), requesting, component)
-
-            plugin_obj = self.loaded_plugins[plugin][version]
-
-            access_name = component
-            if plugin in self.postfix_mappings and component in self.postfix_mappings[plugin] and version in self.postfix_mappings[plugin][component]:
-                access_name = self.postfix_mappings[plugin][component][version]
-            if not hasattr(plugin_obj, access_name):
-                raise RuntimeError("Plugin '%s:%s' dose not have name '%s'" % (plugin, version, access_name) )
-
-            self.loaded_components[component][plugin][version] = getattr(plugin_obj, access_name)
-
-        #record the use of this component, perhaps so the users can save the configuration
-        if not component in self.useing:
-            self.useing[component] = {}
-        if not plugin in self.useing[component]:
-            self.useing[component][plugin] = []
-        if not version in self.useing[component][plugin]:
-            self.useing[component][plugin].append(version)
-
-        self.fire_event('component_loaded', component, requesting, plugin + ":" + version)
-        return self.loaded_components[component][plugin][version]
+        return component
